<?php

class FrontController extends FrontControllerCore
{
    private static function registerPreviewErrorHandler() {
        register_shutdown_function(function () {
            if ($e = error_get_last()) {
                switch ($e['type']) {
                    case E_ERROR:
                    case E_CORE_ERROR:
                    case E_COMPILE_ERROR:
                    case E_PARSE:
                        printf('[PHP_ERROR]%s[PHP_ERROR]', json_encode($e));
                }
            }
        });
    }

    public function init()
    {
        if (Tools::getIsset('theme_name'))
            self::registerPreviewErrorHandler();

        parent::init();
        
        // PrestaShop removes such declaration before class
        self::includeDefines();

        $css_dir = _THEME_CSS_DIR_;
        $img_dir = _THEME_IMG_DIR_;
        $js_dir = _THEME_JS_DIR_;
        if (isPreviewTheme())
        {
            $css_dir = getPreviewCssDir();
            $img_dir = getPreviewImgDir();
            $js_dir = getPreviewJsDir();
        }
        
        $this->context->smarty->assign(array(
            // Usefull for layout.tpl
            'tpl_dir' => getThemeDir(),
            'img_dir' => $img_dir,
            'css_dir' => $css_dir,
            'js_dir' => $js_dir,
            'css_theme_cache' => Configuration::get('PS_CSS_THEME_CACHE'),
            // needed in shopping cart link Billion Themler control - blockuserinfo.php
            'bd_order_process' => Configuration::get('PS_ORDER_PROCESS_TYPE') ? 'order-opc' : 'order',
            'sliderVars' => getSliderModeVars(),
            'templateVars' => getTemplateVars()
        ));
        
        // needed in shopping cart Billion Themler control - blockcart.php
        $param = array('cookie' => $this->context->cookie, 'cart' => $this->context->cart);
        $this->getCartProducts($param);
        // needed to output header search form in Billion Themler theme.
        $this->calculSearchCommon();
    }

    public function displayHeader($display = true)
    {
        parent::displayHeader();
        self::includeDefines();
        $this->smartyOutputContent(getThemeDir().'header.tpl');
    }

    public function displayFooter($display = true)
    {
        // This method will be removed in 1.6
        parent::displayFooter();
        self::includeDefines();
        $this->smartyOutputContent(getThemeDir().'footer.tpl');
    }

    public function display()
    {
        self::includeDefines();
        Tools::safePostVars();

        // assign css_files and js_files at the very last time
        if ((Configuration::get('PS_CSS_THEME_CACHE') || Configuration::get('PS_JS_THEME_CACHE')) && is_writable(_PS_THEME_DIR_.'cache'))
        {
            // CSS compressor management
            if (Configuration::get('PS_CSS_THEME_CACHE'))
                $this->css_files = Media::cccCSS($this->css_files);
            //JS compressor management
            if (Configuration::get('PS_JS_THEME_CACHE') && !$this->useMobileTheme())
                $this->js_files = Media::cccJs($this->js_files);
        }
        
        $this->context->smarty->assign(array(
            'css_files' => $this->css_files,
            'errors' => $this->errors,
            'display_header' => $this->display_header,
            'display_footer' => $this->display_footer,
        ));
        if ((int)implode(explode('.', _PS_VERSION_)) > 1606) { // For versions > 1.6.0.6
            $this->context->smarty->assign(array(
                'js_files' => ($this->getLayout() && (bool)Configuration::get('PS_JS_DEFER')) ? array() : $this->js_files,
                'js_defer' => (bool)Configuration::get('PS_JS_DEFER')
            ));
        } else {
            $this->context->smarty->assign(array(
                'js_files' => $this->getLayout() ? array() : $this->js_files
            ));
        }
        
        $live_edit_content = '';
        // Don't use live edit if on mobile device
        if (!$this->useMobileTheme() && $this->checkLiveEditAccess())
            $live_edit_content = $this->getLiveEditFooter();
        
        $layout = $this->getLayout();
        if ($layout)
        {
            if ($this->template)
                $this->context->smarty->assign('template', $this->context->smarty->fetch($this->template).$live_edit_content);
            else // For retrocompatibility with 1.4 controller
            {
                ob_start();
                $this->displayContent();
                $template = ob_get_contents();
                ob_clean();
                $this->context->smarty->assign('template', $template.$live_edit_content);
            }
            $this->smartyOutputContent($layout);
        }
        else
        {
            Tools::displayAsDeprecated('layout.tpl is missing in your theme directory');
            if ($this->display_header)
                $this->smartyOutputContent(getThemeDir().'header.tpl');

            if ($this->template)
                $this->smartyOutputContent($this->template);
            else // For retrocompatibility with 1.4 controller
                $this->displayContent();

            if ($this->display_footer)
                $this->smartyOutputContent(getThemeDir().'footer.tpl');
        }
                
        return true;
    }

    protected function displayRestrictedCountryPage()
    {
        parent::displayRestrictedCountryPage();
        self::includeDefines();
        $this->smartyOutputContent(getThemeDir().'restricted-country.tpl');
        exit;
    }
    
    public function setMedia()
    {
        // if website is accessed by mobile device
        // @see FrontControllerCore::setMobileMedia()
        if ($this->useMobileTheme())
        {
            $this->setMobileMedia();
            return true;
        }

        self::includeDefines();
        $css_dir = isPreviewTheme() ? getPreviewCssDir() : _THEME_CSS_DIR_;
        $js_dir = isPreviewTheme() ? getPreviewJsDir() : _THEME_JS_DIR_;
        $tpl_dir = getThemeDir();
        $this->addCSS($css_dir.'grid_prestashop.css', 'all');  // retro compat themes 1.5
        $this->addCSS($css_dir.'global.css', 'all');
        $this->addjquery();
        $this->addjqueryPlugin('easing');
        $this->addJS(_PS_JS_DIR_.'tools.js');
        $this->addJS($js_dir.'global.js');

        // Automatically add js files from js/autoload directory in the template
        if (@filemtime($tpl_dir.'js/autoload/'))
            foreach (scandir($tpl_dir.'js/autoload/', 0) as $file)
                if (preg_match('/^[^.].*\.js$/', $file))
                    $this->addJS($tpl_dir.'js/autoload/'.$file);
        // Automatically add css files from css/autoload directory in the template
        if (@filemtime($tpl_dir.'css/autoload/'))
            foreach (scandir($tpl_dir.'css/autoload', 0) as $file)
                if (preg_match('/^[^.].*\.css$/', $file))
                    $this->addCSS($tpl_dir.'css/autoload/'.$file);

        if (Tools::isSubmit('live_edit') && Tools::getValue('ad') && Tools::getAdminToken('AdminModulesPositions'.(int)Tab::getIdFromClassName('AdminModulesPositions').(int)Tools::getValue('id_employee')))
        {
            $this->addJqueryUI('ui.sortable');
            $this->addjqueryPlugin('fancybox');
            $this->addJS(_PS_JS_DIR_.'hookLiveEdit.js');
        }

        if (Configuration::get('PS_QUICK_VIEW'))
            $this->addjqueryPlugin('fancybox');

        if (Configuration::get('PS_COMPARATOR_MAX_ITEM') > 0)
            $this->addJS($js_dir.'products-comparison.js');

        // Execute Hook FrontController SetMedia
        Hook::exec('actionFrontControllerSetMedia', array());
    }

    public function getLayout()
    {
        self::includeDefines();
        $entity = $this->php_self;
        $id_item = (int)Tools::getValue('id_'.$entity);

        $layout_dir = getThemeDir();
        $layout_override_dir  = $this->getOverrideThemeDir();

        $layout = false;
        if ($entity)
        {
            if ($id_item > 0 && file_exists($layout_override_dir.'layout-'.$entity.'-'.$id_item.'.tpl'))
                $layout = $layout_override_dir.'layout-'.$entity.'-'.$id_item.'.tpl';
            elseif (file_exists($layout_override_dir.'layout-'.$entity.'.tpl'))
                $layout = $layout_override_dir.'layout-'.$entity.'.tpl';
        }

        if (!$layout && file_exists($layout_dir.'layout.tpl'))
            $layout = $layout_dir.'layout.tpl';

        return $layout;
    }

    public function setTemplate($default_template)
    {
        self::includeDefines();
        $preview_template = str_replace(_PS_THEME_DIR_, getThemeDir(), $default_template);
        parent::setTemplate($preview_template);
    }

    private static function includeDefines()
    {
        $path = _PS_ROOT_DIR_.'/modules/designerpreview/defines.inc.php';
        if (file_exists($path))
            require_once($path);
    }

    protected function getCartProducts(&$params)
    {
        global $errors;

        // Set currency
        if ((int)$params['cart']->id_currency && (int)$params['cart']->id_currency != $this->context->currency->id)
            $currency = new Currency((int)$params['cart']->id_currency);
        else
            $currency = $this->context->currency;

        $taxCalculationMethod = Group::getPriceDisplayMethod((int)Group::getCurrent()->id);

        $useTax = !($taxCalculationMethod == PS_TAX_EXC);

        $products = $params['cart']->getProducts(true);
        $nbTotalProducts = 0;
        foreach ($products as $product)
            $nbTotalProducts += (int)$product['cart_quantity'];
        $cart_rules = $params['cart']->getCartRules();

        $base_shipping = $params['cart']->getOrderTotal($useTax, Cart::ONLY_SHIPPING);
        $shipping_cost = Tools::displayPrice($base_shipping, $currency);
        $shipping_cost_float = Tools::convertPrice($base_shipping, $currency);
        $wrappingCost = (float)($params['cart']->getOrderTotal($useTax, Cart::ONLY_WRAPPING));
        $totalToPay = $params['cart']->getOrderTotal($useTax);

        if ($useTax && Configuration::get('PS_TAX_DISPLAY') == 1)
        {
            $totalToPayWithoutTaxes = $params['cart']->getOrderTotal(false);
            $this->context->smarty->assign('tax_cost', Tools::displayPrice($totalToPay - $totalToPayWithoutTaxes, $currency));
        }
        
        // The cart content is altered for display
        foreach ($cart_rules as &$cart_rule)
        {
            if ($cart_rule['free_shipping'])
            {
                $shipping_cost = Tools::displayPrice(0, $currency);
                $shipping_cost_float = 0;
                $cart_rule['value_real'] -= Tools::convertPrice($params['cart']->getOrderTotal(true, Cart::ONLY_SHIPPING), $currency);
                $cart_rule['value_tax_exc'] = Tools::convertPrice($params['cart']->getOrderTotal(false, Cart::ONLY_SHIPPING), $currency);
            }
            if ($cart_rule['gift_product'])
            {
                foreach ($products as &$product)
                    if ($product['id_product'] == $cart_rule['gift_product'] && $product['id_product_attribute'] == $cart_rule['gift_product_attribute'])
                    {
                        $product['is_gift'] = 1;
                        $product['total_wt'] = Tools::ps_round($product['total_wt'] - $product['price_wt'], (int)$currency->decimals * _PS_PRICE_DISPLAY_PRECISION_);
                        $product['total'] = Tools::ps_round($product['total'] - $product['price'], (int)$currency->decimals * _PS_PRICE_DISPLAY_PRECISION_);
                        $cart_rule['value_real'] = Tools::ps_round($cart_rule['value_real'] - $product['price_wt'], (int)$currency->decimals * _PS_PRICE_DISPLAY_PRECISION_);
                        $cart_rule['value_tax_exc'] = Tools::ps_round($cart_rule['value_tax_exc'] - $product['price'], (int)$currency->decimals * _PS_PRICE_DISPLAY_PRECISION_);
                    }
            }
        }

        $total_free_shipping = 0;
        if ($free_shipping = Tools::convertPrice(floatval(Configuration::get('PS_SHIPPING_FREE_PRICE')), $currency))
        {
            $total_free_shipping =  floatval($free_shipping - ($params['cart']->getOrderTotal(true, Cart::ONLY_PRODUCTS) + $params['cart']->getOrderTotal(true, Cart::ONLY_DISCOUNTS)));
            $discounts = $params['cart']->getCartRules(CartRule::FILTER_ACTION_SHIPPING);
            if ($total_free_shipping < 0)
                $total_free_shipping = 0;
            if (is_array($discounts) && count($discounts))
                $total_free_shipping = 0;
        }

        $this->context->smarty->assign(array(
            'bd_products' => $products,
            'bd_customizedDatas' => Product::getAllCustomizedDatas((int)($params['cart']->id)),
            'bd_CUSTOMIZE_FILE' => _CUSTOMIZE_FILE_,
            'bd_CUSTOMIZE_TEXTFIELD' => _CUSTOMIZE_TEXTFIELD_,
            'bd_discounts' => $cart_rules,
            'bd_nb_total_products' => (int)($nbTotalProducts),
            'bd_shipping_cost' => $shipping_cost,
            'bd_shipping_cost_float' => $shipping_cost_float,
            'bd_show_wrapping' => $wrappingCost > 0 ? true : false,
            'bd_show_tax' => (int)(Configuration::get('PS_TAX_DISPLAY') == 1 && (int)Configuration::get('PS_TAX')),
            'bd_wrapping_cost' => Tools::displayPrice($wrappingCost, $currency),
            'bd_product_total' => Tools::displayPrice($params['cart']->getOrderTotal($useTax, Cart::BOTH_WITHOUT_SHIPPING), $currency),
            'bd_total' => Tools::displayPrice($totalToPay, $currency),
            'bd_order_process' => Configuration::get('PS_ORDER_PROCESS_TYPE') ? 'order-opc' : 'order',
            'bd_ajax_allowed' => (int)(Configuration::get('PS_BLOCK_CART_AJAX')) == 1 ? true : false,
            'bd_static_token' => Tools::getToken(false),
            'bd_free_shipping' => $total_free_shipping
        ));
        if (count($errors))
            $this->context->smarty->assign('bd_errors', $errors);
        if (isset($this->context->cookie->ajax_blockcart_display))
            $this->context->smarty->assign('bd_colapseExpandStatus', $this->context->cookie->ajax_blockcart_display);
    }

    private function calculSearchCommon()
    {
        $this->context->smarty->assign(array(
            'bd_ENT_QUOTES' =>      ENT_QUOTES,
            'bd_search_ssl' =>      Tools::usingSecureMode(),
            'bd_ajaxsearch' =>      Configuration::get('PS_SEARCH_AJAX'),
            'bd_instantsearch' =>   Configuration::get('PS_INSTANT_SEARCH')
        ));

        return true;
    }
}